// -*- Java -*-
/*!
 * @file  PulseGeneratorImpl.java
 * @brief PulseGenerator
 * @date  $Date$
 *
 * $Id$
 */

package pulsegenerator;

import RTC.TimedLong;
import PGData.EncoderPulse;
import jp.go.aist.rtm.RTC.DataFlowComponentBase;
import jp.go.aist.rtm.RTC.Manager;
import jp.go.aist.rtm.RTC.port.InPort;
import jp.go.aist.rtm.RTC.port.OutPort;
import jp.go.aist.rtm.RTC.util.DataRef;
import RTC.ReturnCode_t;
import pgview.PGView;

/*!
 * @class PulseGeneratorImpl
 * @brief PulseGenerator
 *
 */
public class PulseGeneratorImpl extends DataFlowComponentBase {

  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
	public PulseGeneratorImpl(Manager manager) {  
        super(manager);
        // <rtc-template block="initializer">
        m_m_f_a_val = new TimedLong(new RTC.Time(0,0),0);
        m_m_f_a = new DataRef<TimedLong>(m_m_f_a_val);
        m_Frequency_AIn = new InPort<TimedLong>("Frequency_A", m_m_f_a);
        m_m_f_b_val = new TimedLong(new RTC.Time(0,0),0);
        m_m_f_b = new DataRef<TimedLong>(m_m_f_b_val);
        m_Frequency_BIn = new InPort<TimedLong>("Frequency_B", m_m_f_b);
        m_m_f_c_val = new TimedLong(new RTC.Time(0,0),0);
        m_m_f_c = new DataRef<TimedLong>(m_m_f_c_val);
        m_Frequency_CIn = new InPort<TimedLong>("Frequency_C", m_m_f_c);
        m_m_f_d_val = new TimedLong(new RTC.Time(0,0),0);
        m_m_f_d = new DataRef<TimedLong>(m_m_f_d_val);
        m_Frequency_DIn = new InPort<TimedLong>("Frequency_D", m_m_f_d);
        m_m_pg_data_val = new EncoderPulse(new RTC.Time(0,0),0,0);
        m_m_pg_data = new DataRef<EncoderPulse>(m_m_pg_data_val);
        m_PGcounterOut = new OutPort<EncoderPulse>("PGcounter", m_m_pg_data);
        // </rtc-template>

    }

    /**
     *
     * The initialize action (on CREATED->ALIVE transition)
     * formaer rtc_init_entry() 
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onInitialize() {
        // Registration: InPort/OutPort/Service
        // <rtc-template block="registration">
        // Set InPort buffers
        addInPort("Frequency_A", m_Frequency_AIn);
        addInPort("Frequency_B", m_Frequency_BIn);
        addInPort("Frequency_C", m_Frequency_CIn);
        addInPort("Frequency_D", m_Frequency_DIn);
        
        // Set OutPort buffer
        addOutPort("PGcounter", m_PGcounterOut);
        // </rtc-template>
        m_view.init();
        return super.onInitialize();
    }

    /***
     *
     * The finalize action (on ALIVE->END transition)
     * formaer rtc_exiting_entry()
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onFinalize() {
//        return super.onFinalize();
//    }

    /***
     *
     * The startup action when ExecutionContext startup
     * former rtc_starting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStartup(int ec_id) {
//        return super.onStartup(ec_id);
//    }

    /***
     *
     * The shutdown action when ExecutionContext stop
     * former rtc_stopping_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onShutdown(int ec_id) {
//        return super.onShutdown(ec_id);
//    }

    /***
     *
     * The activated action (Active state entry action)
     * former rtc_active_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onActivated(int ec_id) {
//        return super.onActivated(ec_id);
//    }

    /***
     *
     * The deactivated action (Active state exit action)
     * former rtc_active_exit()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onDeactivated(int ec_id) {
//        return super.onDeactivated(ec_id);
//    }

    /***
     *
     * The execution action that is invoked periodically
     * former rtc_active_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onExecute(int ec_id) {
    	if(m_view.isStarting()){
    		long fre = m_view.getFrequencyA();
    		m_r_counter += fre;
    		m_m_pg_data_val.right_encoder = (int)m_r_counter;
			System.out.println(m_r_counter);
			
    		fre = m_view.getFrequencyB();
    		m_l_counter += fre;
    		m_m_pg_data_val.left_encoder = (int)m_l_counter;
			System.out.println(m_l_counter);
			
    		m_PGcounterOut.write();
    	}
    	if(m_Frequency_AIn.isNew() && m_Frequency_BIn.isNew()){
    		m_Frequency_AIn.read();
    		m_Frequency_BIn.read();
    		m_r_counter += (int)m_m_f_a.v.data;
    		m_m_pg_data_val.right_encoder = (int)m_r_counter;
			System.out.println(m_r_counter);

			m_l_counter += (int)m_m_f_b.v.data;
    		m_m_pg_data_val.left_encoder = (int)m_l_counter;
			System.out.println(m_l_counter);

			m_PGcounterOut.write();
    		
    	}
        return super.onExecute(ec_id);
    }

    /***
     *
     * The aborting action when main logic error occurred.
     * former rtc_aborting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//  @Override
//  public ReturnCode_t onAborting(int ec_id) {
//      return super.onAborting(ec_id);
//  }

    /***
     *
     * The error action in ERROR state
     * former rtc_error_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    public ReturnCode_t onError(int ec_id) {
//        return super.onError(ec_id);
//    }

    /***
     *
     * The reset action that is invoked resetting
     * This is same but different the former rtc_init_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onReset(int ec_id) {
//        return super.onReset(ec_id);
//    }

    /***
     *
     * The state update action that is invoked after onExecute() action
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStateUpdate(int ec_id) {
//        return super.onStateUpdate(ec_id);
//    }

    /***
     *
     * The action that is invoked when execution context's rate is changed
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onRateChanged(int ec_id) {
//        return super.onRateChanged(ec_id);
//    }
//
    // DataInPort declaration
    // <rtc-template block="inport_declare">
    protected TimedLong m_m_f_a_val;
    protected DataRef<TimedLong> m_m_f_a;
    /*!
     */
    protected InPort<TimedLong> m_Frequency_AIn;

    protected TimedLong m_m_f_b_val;
    protected DataRef<TimedLong> m_m_f_b;
    /*!
     */
    protected InPort<TimedLong> m_Frequency_BIn;

    protected TimedLong m_m_f_c_val;
    protected DataRef<TimedLong> m_m_f_c;
    /*!
     */
    protected InPort<TimedLong> m_Frequency_CIn;

    protected TimedLong m_m_f_d_val;
    protected DataRef<TimedLong> m_m_f_d;
    /*!
     */
    protected InPort<TimedLong> m_Frequency_DIn;

    
    // </rtc-template>

    // DataOutPort declaration
    // <rtc-template block="outport_declare">
    protected EncoderPulse m_m_pg_data_val;
    protected DataRef<EncoderPulse> m_m_pg_data;
    /*!
     */
    protected OutPort<EncoderPulse> m_PGcounterOut;

    
    // </rtc-template>

    // CORBA Port declaration
    // <rtc-template block="corbaport_declare">
    
    // </rtc-template>

    // Service declaration
    // <rtc-template block="service_declare">
    
    // </rtc-template>

    // Consumer declaration
    // <rtc-template block="consumer_declare">
    
    // </rtc-template>


    protected PGView m_view = new PGView(); 

    private long m_r_counter = 0;
    private long m_l_counter = 0;

}
